//***********************************************************
//
//  section : NXV4 package
//
//  code    : DXF class _____________________________________
//            
//                         Morelet wavelet digital filter
//                             at given frequency f0
//                      _____________________________________
//
//  CTOR's
//  ----
//
//  dxf filter(f0d, sfd, file_ip, file_op, cmd)
//              ^    ^    ^        ^        ^
//              |    |    |        |        |_ command
//              |    |    |        |
//              |    |    |        |_ o/p file
//              |    |    |
//              |    |    |_ i/p file
//              |    |
//              |    |_ sigma_omega * delta
//              |
//              |_ f0 * delta
//
//
//  dxf filter(f0d, sfd, ip, c_op, s_op, N, cmd)
//              ^    ^    ^   ^     ^    ^   ^
//              |    |    |   |     |    |   |_ command
//              |    |    |   |     |    |
//              |    |    |   |     |    |_ nr. bins
//              |    |    |   |     |
//              |    |    |   |     |_ sin o/p
//              |    |    |   |
//              |    |    |   |_ cos o/p
//              |    |    |
//              |    |    |_ i/p array
//              |    |
//              |    |_ sigma_omega * delta
//              |
//              |_ f0 * delta
//
//    COMMAND _______________________________________________
//
//                        apodis./space  spec.leak  sgm  lobe
//                       ______________  _________  ___  ____
//    classical
//      PS               Poisson /TIME :  -6dB/oct|1.10|-20dB
//      GX               Gauss    -"-  :  -6  -"- |1.50|-42dB
//      HM               Hamming  -"-  :  -6  -"- |1.50|-42dB
//      O4               O4       -"-  :  -6  -"- |1.70|-52dB
//      BA               Blackman -"-  : -18  -"- |1.90|-60dB
//                       ____________________________________
//    FoxLima
//      F0               F0 / NATIVE   :  -6dB/oct|1.00|-13dB
//      F1               F1 / FOURIER  : -12  -"- |1.30|-22dB
//      F2               F2 /   -"-    : -18  -"- |1.60|-28dB
//      F3               F3 /   -"-    : -24  -"- |1.80|-34dB
//      F4               F4 /   -"-    : -30  -"- |2.00|-40dB
//      F5               F5 /   -"-    : -36  -"- |2.20|-44dB
//      F6               F6 /   -"-    : -42  -"- |2.40|-48dB
//      F7               F7 /   -"-    : -48  -"- |2.60|-52dB
//      F8               F8 /   -"-    : -54  -"- |2.80|-56dB
//      F9               F9 /   -"-    : -60  -"- |3.00|-60dB
//                       ____________________________________
//
//
//  authors : Mihai-Tiberiu, Maria - Dima
//  leader  : Yu.N. Pepelyshev 
//
//                              _____________________________
//
//                              Nuclear Security Group IBR-2M
//
//                       Joint Institute for Nuclear Research
//                                             Dubna - Russia
//                       ____________________________________
//
//  date    : NXV4/v1.0.alfa   / Fri Feb 24 15:27:32 MSK 2023
//
//***********************************************************


#pragma once


#include <iostream>
#include <string>

using std::cout                                             ;
using std::endl                                             ;
using std::string                                           ;


namespace dxf4
{
//===========================================================

template <class T>
concept scalar = requires(int i, float f, double d, long double l, T b, T c)
{ i*c; 
  f*c;
  d*c;
  l*c;
  b+c;
};


 template<typename scalar=double>
 class dxf
   {public: double      f0d = 1.0                           ; 
            double      sfd = 0.1                           ; 
            int         FC  = 0                             ; 
            string      opx                                 ; 

      // ____________________________________________________

      dxf()                   // plain     
       {N = 0                                               ;
        file_ip = " " ; file_op = " "                       ;} 


      dxf(double, double, string , string , string )        ;
      dxf(double, double, scalar*, scalar*, scalar*,
                                               int, string );

      
      dxf(const dxf& A)       // copy       
       {f0d = A.f0d; sfd = A.sfd                            ;
        N = A.N                                             ;}

      // ____________________________________________________

      dxf& operator=(const dxf& A)
       {f0d = A.f0d; sfd = A.sfd                            ;
        N = A.N                                             ;
        return const_cast<dxf<scalar>&>(A)                  ;} 

      // ____________________________________________________

      friend std::ostream& operator<<(std::ostream&, 
                                      dxf<auto> const&    ) ;

      // ____________________________________________________

     ~dxf()
       {                                                    ;}

      // ____________________________________________________

    private: int    N                                       ; 
             string file_ip, file_op                        ;

 };
//
//===========================================================
//
};
